## Code for estimation of tag-jump ratio

## Input data = 4 Excel files with OTU tables
## OTUs = rows, samples = columns
## First colums = "OTU" contains OTU ID
## Sample names should have suffix with platform name, e.g. "G5300DNB" and "G5300Nova"


library(data.table)
library(brms)
library(plyr)
library(ggplot2)
library(openxlsx)

set.seed(14789)
theme_set(theme_classic(base_size = 14))

## Set working directory
# setwd("~")


## Function to load and prepare data
prep_data <- function(x){
  # x = file name

  OTUTab <- readxl::read_excel(path = x, na = c("", " ", "NA"))
  setDT(OTUTab)

  ############### Split data by platform

  samps_DNB <- c("OTU", grep(pattern = "DNB$", x = colnames(OTUTab), value = T))
  samps_Nova <- c("OTU", grep(pattern = "Nova$", x = colnames(OTUTab), value = T))

  DT_Nova <- OTUTab[, ..samps_Nova]
  DT_DNB <- OTUTab[, ..samps_DNB]

  DT_Nova <- DT_Nova[rowSums(DT_Nova[,-1]) > 0]
  DT_DNB <- DT_DNB[rowSums(DT_DNB[,-1]) > 0]

  ## Reshape data to long format
  DT_Nova_ln <- melt(data = DT_Nova, id.vars = "OTU", variable.name = "SampleID", value.name = "Abundance")
  DT_DNB_ln <- melt(data = DT_DNB, id.vars = "OTU", variable.name = "SampleID", value.name = "Abundance")

  DT_Nova_ln <- DT_Nova_ln[ Abundance > 0 ]
  DT_DNB_ln <- DT_DNB_ln[ Abundance > 0 ]

  ## Estimate total OTU abundances
  OtuTot_Nova <- DT_Nova_ln[ , .(Total = sum(Abundance)), by = "OTU"]
  OtuTot_DNB <- DT_DNB_ln[ , .(Total = sum(Abundance)), by = "OTU"]

  ## Add total abundances to the table
  DT_Nova_ln <- merge(x = DT_Nova_ln, y = OtuTot_Nova, by = "OTU", all.x = T)
  DT_DNB_ln <- merge(x = DT_DNB_ln, y = OtuTot_DNB, by = "OTU", all.x = T)

  
  ############### Esimate UNCROSS scores

  ## UNCROSS score
  uncross_score <- function(x, N, n, f = 0.01, tmin = 0.1){
    # x = OTU abundance in a sample
    # N = total OTU abundance
    # n = number of samples
    # f = expected cross-talk rate, e.g. 0.01
    # tmin = min score to be considered as cross-talk
    z <- f * N / n             # Expected treshold
    sc <- 2 / (1 + exp(x/z))   # t-score
    res <- data.frame(Score = sc, TagJump = sc >= tmin)
    return(res)
  }

  DT_Nova_ln <- cbind(
    DT_Nova_ln,
    uncross_score(
      x = DT_Nova_ln$Abundance,
      N = DT_Nova_ln$Total,
      n = length(unique(DT_Nova_ln$SampleID)),
      f = 0.01)
    )

  DT_DNB_ln <- cbind(
    DT_DNB_ln,
    uncross_score(
      x = DT_DNB_ln$Abundance,
      N = DT_DNB_ln$Total,
      n = length(unique(DT_DNB_ln$SampleID)),
      f = 0.01)
    )

  ## Number of tag-jumps
  sum(DT_Nova_ln$TagJump)
  sum(DT_DNB_ln$TagJump)

  ## Merge data
  TGJ <- rbind(
    data.frame(Platform = "NovaSeq", DT_Nova_ln),
    data.frame(Platform = "DNBseq", DT_DNB_ln)
    )

  ## Extract sample ID withou platform suffix
  TGJ$SampleBiol <- gsub(pattern = "Nova$|DNB$", replacement = "", x = TGJ$SampleID)

  setDT(TGJ)
  TGJ[ TagJump == FALSE, TagJump_type := "not_tagjump" ]
  TGJ[ TagJump == TRUE, TagJump_type := "tagjump" ]

  return(TGJ)
}


## Load all OTU tables
TGJ_raw <- prep_data("OTU_matrix1.xlsx")
TGJ_sng <- prep_data("OTU_matrix2.xlsx")
TGJ_met <- prep_data("OTU_matrix3.xlsx")
TGJ_rar <- prep_data("OTU_matrix4.xlsx")



## Summary stats
count_tags <- function(TGJ){
  
  ## Number of OTU-sample combinations
  rz <- as.data.frame(table(TGJ$TagJump_type, TGJ$Platform))
  colnames(rz) <- c("TagType", "Platform", "Count")
  rz <- data.frame(Parameter = "OTU_sample_combinations", rz)

  ## Number of reads
  nova_tj <- TGJ[ Platform %in% "NovaSeq" & TagJump_type %in% "tagjump" ]
  nova_tj <- sum(nova_tj$Abundance)
  nova_tot <-sum(TGJ[ Platform %in% "NovaSeq" ]$Abundance)

  dnb_tj <- TGJ[ Platform %in% "DNBseq" & TagJump_type %in% "tagjump" ]
  dnb_tj <- sum(dnb_tj$Abundance)
  dnb_tot <-sum(TGJ[ Platform %in% "DNBseq" ]$Abundance)

  rz2 <- rbind(
  data.frame(TagType = "all_reads", Platform = "NovaSeq", Count = nova_tot),
  data.frame(TagType = "tag-jumps", Platform = "NovaSeq", Count = nova_tj),
  data.frame(TagType = "tag-jumps_perc", Platform = "NovaSeq", Count = round(nova_tj / nova_tot * 100, 3)),

  data.frame(TagType = "all_reads", Platform = "DNBseq", Count = dnb_tot),
  data.frame(TagType = "tag-jumps", Platform = "DNBseq", Count = dnb_tj),
  data.frame(TagType = "tag-jumps_perc", Platform = "DNBseq", Count = round(dnb_tj / dnb_tot * 100, 3))
  )

  rz2 <- data.frame(Parameter = "Reads", rz2)

  rz <- rbind(rz, rz2)
  rzw <- reshape2::dcast(data = rz, formula = Parameter + TagType ~ Platform, value.var = "Count")
  return(rzw)
}

## Summary for each dataset
TJ_raw <- count_tags(TGJ_raw)
TJ_sng <- count_tags(TGJ_sng)
TJ_met <- count_tags(TGJ_met)
TJ_rar <- count_tags(TGJ_rar)

## Concatenate results into a single table
TJ <- rbind(
  data.frame(Data = "Raw", TJ_raw),
  data.frame(Data = "No_singletons", TJ_sng),
  data.frame(Data = "No_singletons,Metazoa_only", TJ_met),
  data.frame(Data = "Rarefied,Metazoa_only", TJ_rar)
  )

## Export results
write.xlsx(list(
  "tagjump" = TJ
  ),
  file = "TagJump_Counts.xlsx", colNames = TRUE)



######################### N OTUs in filtered and unfiltered data


## Estimate number of tag-jump OTUs per sample
otu_jumps <- function(TGJ){

  ## Count number of OTUs in filtered and unfiltered data for each sample
  Nova_OTU_filt <- TGJ[
       Platform %in% "NovaSeq" & TagJump == FALSE,
       .(Abundance = sum(Abundance), Richness = .N),
       by=.(SampleID, SampleBiol, Platform) ]

  DNB_OTU_filt <- TGJ[
       Platform %in% "DNBseq" & TagJump == FALSE,
       .(Abundance = sum(Abundance), Richness = .N),
       by=.(SampleID, SampleBiol, Platform) ]

  ## Count number of OTUs in unfiltered data for each sample
  Nova_OTU_unfilt <- TGJ[
       Platform %in% "NovaSeq",
       .(Abundance = sum(Abundance), Richness = .N),
       by=.(SampleID, SampleBiol, Platform) ]

  DNB_OTU_unfilt <- TGJ[
       Platform %in% "DNBseq",
       .(Abundance = sum(Abundance), Richness = .N),
       by=.(SampleID, SampleBiol, Platform) ]

  ## Merge data
  OTU_counts <- rbind(
    data.table(Data = "filtered", Nova_OTU_filt),
    data.table(Data = "unfiltered", Nova_OTU_unfilt),
    data.table(Data = "filtered", DNB_OTU_filt),
    data.table(Data = "unfiltered", DNB_OTU_unfilt)
    )

  OTU_counts$Richness <- as.numeric(OTU_counts$Richness)

  ## Reshape
  OTU_counts_w <- dcast(
    data = melt(
      data = OTU_counts, id.vars = c("Data", "SampleBiol", "Platform"),
      measure.vars = c("Abundance", "Richness"),
      variable.name = "Param", value.name = "Value"),
    formula = SampleBiol ~ Platform + Data + Param, fill = 0, value.var = "Value")

  return(OTU_counts_w)
}


OTU_counts_raw <- otu_jumps(TGJ_raw)
OTU_counts_sng <- otu_jumps(TGJ_sng)
OTU_counts_met <- otu_jumps(TGJ_met)
OTU_counts_rar <- otu_jumps(TGJ_rar)


## Export results
write.xlsx(list(
  "raw" = OTU_counts_raw,
  "no_singl" = OTU_counts_sng,
  "metazoa" = OTU_counts_met,
  "raref_metazoa" = OTU_counts_rar
  ),
  file = "OTU_Counts_TJFilt_AllTJs.xlsx", colNames = TRUE)



######################### model-based tag-jump ratio

## Prepare data for model
prep_dat <- function(x){
  # x = result of otu_jumps

  ## Estimate number of tag-jumps
  x$DNBSeq_Delta_Abundance <- with(x, DNBseq_unfiltered_Abundance - DNBseq_filtered_Abundance)
  x$DNBSeq_Delta_OTU <- with(x, DNBseq_unfiltered_Richness - DNBseq_filtered_Richness)
  x$NovaSeq_Delta_Abundance <- with(x, NovaSeq_unfiltered_Abundance - NovaSeq_filtered_Abundance)
  x$NovaSeq_Delta_OTU <- with(x, NovaSeq_unfiltered_Richness - NovaSeq_filtered_Richness)

  ## Reshape data
  DNB_abu <- data.table(ParamType = "Abundance", Platform = "DNBseq",
    melt(data = x, id.vars = c("SampleBiol", "DNBseq_unfiltered_Abundance"),
      measure.vars = "DNBSeq_Delta_Abundance", variable.name = "Param", value.name = "Value"))

  DNB_otu <- data.table(ParamType = "OTU", Platform = "DNBseq",
    melt(data = x, id.vars = c("SampleBiol", "DNBseq_unfiltered_Richness"),
      measure.vars = "DNBSeq_Delta_OTU", variable.name = "Param", value.name = "Value"))

  Nov_abu <- data.table(ParamType = "Abundance", Platform = "NovaSeq",
    melt(data = x, id.vars = c("SampleBiol", "NovaSeq_unfiltered_Abundance"),
      measure.vars = "NovaSeq_Delta_Abundance", variable.name = "Param", value.name = "Value"))

  Nov_otu <- data.table(ParamType = "OTU", Platform = "NovaSeq",
    melt(data = x, id.vars = c("SampleBiol", "NovaSeq_unfiltered_Richness"),
      measure.vars = "NovaSeq_Delta_OTU", variable.name = "Param", value.name = "Value"))

  ## Merge
  colnames(DNB_abu)[4] <- colnames(DNB_otu)[4] <- colnames(Nov_abu)[4] <- colnames(Nov_otu)[4] <- "Total"
  res <- rbind(DNB_abu, DNB_otu, Nov_abu, Nov_otu)

  return(res)
}


DT_mod_raw <- prep_dat(OTU_counts_raw)
DT_mod_sng <- prep_dat(OTU_counts_sng)
DT_mod_met <- prep_dat(OTU_counts_met)
DT_mod_rar <- prep_dat(OTU_counts_rar)


## Bayesian generalized linear mixed model with binomial errors and logit link,
# where ‘sample’ is used as a random effect
est_tj_rate <- function(x){
  md <- brm(Value | trials(Total) ~ Platform + (1 | SampleBiol), 
   data = x,
   family = binomial("logit"),
   chains = 7, cores = 7,
   iter = 15000, warmup = 2000)
  return(md)
}


## Fit models to both - number of OTUs and number of reads
MD_raw <- dlply(.data = DT_mod_raw, .variables = "ParamType", .fun = est_tj_rate)
MD_sng <- dlply(.data = DT_mod_sng, .variables = "ParamType", .fun = est_tj_rate)
MD_met <- dlply(.data = DT_mod_met, .variables = "ParamType", .fun = est_tj_rate)
MD_rar <- dlply(.data = DT_mod_rar, .variables = "ParamType", .fun = est_tj_rate)


## Model summaries
MD_raw
MD_sng
MD_met
MD_rar

