### Bioinformatics

#1. Demultiplexing paired-end reads. Input = multiplexed raw R1.fastq and R2.fastq files.
#cutadapt v3.4 (https://cutadapt.readthedocs.io/en/stable/guide.html#demultiplexing)

# Round1 demux
cutadapt -g file:IndexesFileForDemux.fasta \
-e 1 --no-indels --overlap 12 \
--minimum-length 20 \
-o demux_samples/round1-{name}.R1.fq \
-p demux_samples/round1-{name}.R2.fq \
input_R1.fq input_R2.fq

#Round2 demux (RC; R1 and R2 position switched!)
cutadapt -g file:IndexesFileForDemux.fasta \
-e 1 --no-indels --overlap 12 \
--minimum-length 20 \
-o round2-{name}.R2.fq -p round2-{name}.R1.fq \
round1-unknown.R2.fq round1-unknown.R1.fq

#merge demultiplexed reads per sample from round1 and round2
ls | grep "R1.fq" | grep "round1-" | grep -v "unknown" > demux_R1_list.txt
while read DEMUXFILES; do
    R1=$(echo $DEMUXFILES | sed -e 's/round1-//' )
    R2=$(echo $DEMUXFILES | sed -e 's/round1-//' | sed -e 's/R1/R2/' )
    if [ -f round2-$R1 ]; then
        cat round1-$R1 round2-$R1 > $R1
        rm round1-$R1
        rm round2-$R1
    else
        echo "round2-$R1 does not exist"
    fi
    if [ -f round2-$R2 ]; then
        cat round1-$R2 round2-$R2 > $R2
        rm round1-$R2
        rm round2-$R2
    else
        echo "round2-$R2 does not exist"
    fi
done < demux_R1_list.txt && rm demux_R1_list.txt

#Demultiplexed DNBSEQ-G400RS (2x200 bp) and NovaSeq 6000 (2x250 bp) data are deposited in the Sequence Read Archive (SRA) under BioProject ID PRJNA743174.



### Note that the following processes denote per sample treatments. Loop scripts to process all samples at once are not provided here.
#2.	Reorienting paired-end reads to 5’-3’ as based on primer sequences
#fqgrep (https://github.com/indraniel/fqgrep)

#specify forward and reverse primers. Degenerate bases in between []
fwd_primer=$"GG[AT]AC[AT]GG[AT]TGAAC[AT]GT[AT]TA[CT]CC[CT]CC"
rev_primer=$"TA[ATGC]AC[CT]TC[ATGC]GG[AG]TG[ATGC]CC[AG]AA[AG]AA[CT]CA"

## reorient R1 reads
#seach fwd primer and write to file; -m 2 to allow 2 mismatches
fqgrep -m 2 -p $fwd_primer -e sample1_R1.fq > sample1_R1.5_3.fastq

#search rev primer and write to file; -m 2 to allow 2 mismatches
fqgrep -m 2 -p $rev_primer -e sample1_R1.fq > sample1_R1.3_5.fastq 

#if rev primer found, then make reverse complementary and merge with 5_3.fastq file
fastx_reverse_complement -Q33 -i sample1_R1.3_5.fastq >> sample1_R1.5_3.fastq

## reorient R2 reads
#seach fwd primer and write to file; -m 2 to allow 2 mismatches
fqgrep -m 2 -p $fwd_primer -e sample1_R2.fq > sample1_R2.5_3.fastq 

#search rev primer and write to file; -m 2 to allow 2 mismatches
fqgrep -m $mismatches -p $rev_primer -e sample1_R2.fq > sample1_R2.3_5.fastq 

#if rev fwd found, then make reverse complementary and merge with 3_5.fastq file
  #for paired-end reads merging, R2 has to be 3'-5' oriented.
fastx_reverse_complement -Q33 -i sample1_R2.5_3.fastq >> sample1_R2.3_5.fastq


#3.	Assemble paired-end data
#vsearch v2.17.0 (https://github.com/torognes/vsearch)
vsearch --fastq_mergepairs sample1_R1.5_3.fastq \
--reverse sample1_R2.3_5.fastq \
--fastq_minovlen 10 \
--fastq_minmergelen 10 \
--fastq_allowmergestagger \
--fastq_maxdiffs 20 \
--fastq_maxns 0 \
--fastq_maxmergelen 600 \
--fastqout sample1_merged.fq

#4.	Clip primers from merged reads (cutadapt v3.4)
 #based on primers specified in liked_fwd_revRC_primers.fasta file. 
 #Linked primres = "GGWACWGGWTGAACWGTWTAYCCYCC...TGRTTYTTYGGICAYCCIGARGTITA"

cutadapt \
--revcomp \
-e 2 \
--minimum-length 32 \
--overlap 24 \
--no-indels \
--untrimmed-output sample1_merged.untrimmed.fq \
-g file:liked_fwd_revRC_primers.fasta \
-o sample1_merged.primersCut.fq sample1_merged.fq


#5.	Quality filtering
#trimmomatic v0.39 (https://github.com/usadellab/Trimmomatic)

trimmomatic SE \
sample1_merged.primersCut.fq \
sample1_merged.primersCut.qualFilt.fq \
-phred33 \
LEADING:11 TRAILING:11 \
SLIDINGWINDOW:5:30 \
MINLEN:32

#6.	Chimera filtering
#vsearch v2.17.0 (https://github.com/torognes/vsearch)

#dereplicate sequences
vsearch \
--derep_fulllength sample1_merged.primersCut.qualFilt.fasta \
--minuniquesize 1 --sizeout \
--fasta_width 0 \
--uc dereplicated.uc \
--output $input.derep.fasta

#pre-cluster sequences; sorts seqs automaticcaly by decreasing abundance
vsearch \
--cluster_size $input.derep.fasta \
--id 097 --strand both \
--sizein --sizeout \
--fasta_width 0 \
--uc tempdir/preclustered.uc \
--centroids $input.preclustered.fasta

#search chimeras
vsearch \
--uchime_denovo $input.preclustered.fasta \
--abskew 2 --minh 0.28 --sizein --sizeout \
--fasta_width 0 \
--chimeras $input.denovo.chimeras.fasta \
--nonchimeras $input.denovo.nonchimeras.fasta

#final output; extract all non-chimeric sequences
vsearch \
--usearch_global sample1_merged.primersCut.qualFilt.fasta \
-db $input.denovo.nonchimeras.fasta \
--sizein --xsize \
--id 0.97 \
--strand both \
--fasta_width 0 \
--matched sample1.denovo.nonchimeras.fasta


#7.	Clustering
#vsearch v2.17.0 (https://github.com/torognes/vsearch)

#global dereplication (all chimera filtered samples)
find . -maxdepth 1 -name "*.fasta" | parallel -j 1 "cat {}" \
| vsearch --derep_fulllength - \
--output - \
--fasta_width 0 \
--threads 1 \
--sizein --sizeout > Glob_derep.fasta

#clustering
vsearch \
--cluster_size Glob_derep.fasta \
--id 0.97 --iddef 2 \
--centroids OTUs.fasta \
--fasta_width 0 \
--sizein --sizeout

### Dereplication of individual samples, add sample ID to the header
derep_rename () {
samp_name=$(basename $1 | awk 'BEGIN{FS="."} {$NF=""; print $0}' | sed 's/ //g')
  vsearch --derep_fulllength "$1" --output - \
  --fasta_width 0 \
 --sizein --sizeout \
  | sed 's/>.*/&;sample='"$samp_name"'/' > tempdir/"$samp_name".fasta
}
export -f derep_rename
find . -maxdepth 1 -name "*.fasta" | parallel -j 1 "derep_rename {}"

#merge all dereplicated samples
cat tempdir/*.fasta > tempdir/Dereplicated_samples.fasta

#OTU table creation
vsearch \
--usearch_global tempdir/Dereplicated_samples.fasta \
--db OTUs.fasta \
--id 0.97 \
--sizein --sizeout \
--otutabout OTU_table.txt



#############################################################################
#############################################################################
# ASVs workflow with DADA2
    #input fastq files = demultiplexed, reoriented and primers clipped per-sample files!

#open R
R
library(dada2) #ver 1.18

#path the input sequences
path <- "home/xxx/"
#Results path
path_results <- file.path(path, "results_DADA2")
if(!dir.exists(path_results)) dir.create(path_results)
#patterns to specify R1 and R2 files
file_pattern <- c("F" = "_R1.fq", "R" = "_R2.fq")

#sample names
path_process <- path
fas_Fs_process <- sort(list.files(path_process, pattern = file_pattern["F"], full.names = TRUE))
fas_Rs_process <- sort(list.files(path_process, pattern = file_pattern["R"], full.names = TRUE))
sample_names <- sapply(strsplit(basename(fas_Fs_process), "_"), function(x) x[1])


        #inspect quality profiles
        pdf(file.path(path_results, "Read_quality_profile_aggregated_F.pdf"))
        p <- plotQualityProfile(sample(fas_Fs_process, replace = FALSE, size = ifelse(length(fas_Fs_process) < 100, length(fas_Fs_process), 100)), aggregate = TRUE)
        p + ggplot2::labs(title = "Forward")
        dev.off()
        
        pdf(file.path(path_results, "Read_quality_profile_aggregated_R.pdf"))
        p <- plotQualityProfile(sample(fas_Rs_process, replace = FALSE, size = ifelse(length(fas_Rs_process) < 100, length(fas_Rs_process), 100)), aggregate = TRUE)
        p + ggplot2::labs(title = "Reverse")
        dev.off()


#quality filtering
fas_Fs_filtered <- file.path(path, "filtered", basename(fas_Fs_process))
fas_Rs_filtered <- file.path(path, "filtered", basename(fas_Rs_process))
all.equal(basename(fas_Fs_process), basename(fas_Fs_filtered))
names(fas_Fs_filtered) <- sample_names
names(fas_Rs_filtered) <- sample_names
qfilt <- filterAndTrim(fas_Fs_process, fas_Fs_filtered, fas_Rs_process, fas_Rs_filtered, maxN = 0, maxEE = c(1, 1), truncQ = 2, compress = TRUE, multithread = TRUE)
write.csv(qfilt, file.path(path_results, "qfilt_report.csv"), row.names = TRUE)

#learn the error rates
errors_F <- learnErrors(fas_Fs_filtered, multithread = TRUE, randomize = TRUE)
errors_R <- learnErrors(fas_Rs_filtered, multithread = TRUE, randomize = TRUE)

#plot errors
pdf(file.path(path_results, "Error_rates_learning_F.pdf"))
p <- plotErrors(errors_F, nominalQ = TRUE)
p + ggplot2::labs(title = "Error Forward")
dev.off()
pdf(file.path(path_results, "Error_rates_learning_R.pdf"))
p <- plotErrors(errors_R, nominalQ = TRUE)
p + ggplot2::labs(title = "Error Reverse")
dev.off()

#dereplicate, merge paired-end reads
merged_list <- vector("list", length(sample_names))
names(merged_list) <- sample_names

for(i in sample_names){
  cat("Processing -", which(sample_names == i), "/", length(sample_names), "-", i, "\n")
  derep_Fs <- derepFastq(fas_Fs_filtered[[i]], verbose = TRUE)
  derep_Rs <- derepFastq(fas_Rs_filtered[[i]], verbose = TRUE)
  dds_Fs <- dada(derep_Fs, err = error_F, multithread = TRUE, verbose = TRUE)
  dds_Rs <- dada(derep_Rs, err = error_R, multithread = TRUE, verbose = TRUE)
  merged_list[[i]] <- mergePairs(dds_Fs, derep_Fs, dds_Rs, derep_Rs, verbose = TRUE)
}

#make ASVs tabl
ASV_tab <- makeSequenceTable(merged_list)
#remove chimeras
ASV_tab_nochim <- removeBimeraDenovo(ASV_tab, method = "consensus", multithread = TRUE, verbose = TRUE)


#############################################################################
#############################################################################
###LULU postclustering of ASVs (97%)

#specify input OTUs/ASVs fasta file
input_fasta=$"ASVs.fasta"
#make blast db
makeblastdb -in $input_fasta -parse_seqids -dbtype nucl
#generate match_list.txt
blastn -db $input_fasta \
-outfmt '6 qseqid sseqid pident' \
-out match_list.txt \
-qcov_hsp_perc 80 \
-perc_identity 90 \
-query $input_fasta

#Run LULU in R
R
library(devtools)
setwd("/home/LULU_ASVs97")
otutable_name <- read.table("ASVs_table.txt", header=T, row.names=1)
matchlist_name <- read.table("match_list.txt")
curated_result <- lulu::lulu(otutable_name, matchlist_name, minimum_ratio_type = "min", minimum_match = 97)

#discarded/merged ASVs
curated_result$discarded_otus

write.table(curated_result$curated_table, file ="LULU.ASV_table.txt")
